﻿// <copyright file="JsonFunctionsTest.cs" company="Felipe José Angriman">
//   Copyright 2009 Felipe José Angriman 
//
//   Licensed under the Apache License, Version 2.0 (the "License"); 
//   you may not use this file except in compliance with the License. 
//   You may obtain a copy of the License at 
//   
//       http://www.apache.org/licenses/LICENSE-2.0 
// 
//   Unless required by applicable law or agreed to in writing, software
//   distributed under the License is distributed on an "AS IS" BASIS, 
//   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//   See the License for the specific language governing permissions and 
//   limitations under the License. 
// </copyright>
// <author>Felipe José Angriman</author>
// <summary>Defines the JsonFunctionsTest Class</summary>

namespace Core.Tests
{
    using System;
    using Core.Json;
    using Xunit;

    public class JsonFunctionsTest
    {
        public JsonFunctionsTest()
        {
        }

        [Fact()]
        public void TestJSONObjectSerialization()
        {
            Assert.Equal("{}", JsonFunctions.Serialize(new object()));
            Assert.Equal("{\"Prop1\":1,\"Prop2\":\"\"}", JsonFunctions.Serialize(new { Prop1 = 1, Prop2 = string.Empty }));
            Assert.Equal("{\"Prop1\":1,\"Prop2\":\"Hello there!\",\"NestedArray\":[4,8,\"\"]}", JsonFunctions.Serialize(new { Prop1 = 1, Prop2 = "Hello there!", NestedArray = new object[] { 4, 8, string.Empty } }));
        }

        [Fact()]
        public void TestJSONIEnumerableSerialization()
        {
            Assert.Equal("[]", JsonFunctions.Serialize(new object[0]));
            Assert.Equal("[1,\"Hello there!\",\"c\"]", JsonFunctions.Serialize(new object[] { 1, "Hello there!", 'c' }));
        }

        [Fact()]
        public void TestJSONNullValueSerialization()
        {
            Assert.Equal("null", JsonFunctions.Serialize(null));
            Assert.Equal("null", JsonFunctions.Serialize(DBNull.Value));
        }

        [Fact()]
        public void TestJSONBooleanSerialization()
        {
            Assert.Equal("true", JsonFunctions.Serialize(true));
            Assert.Equal("false", JsonFunctions.Serialize(false));
        }

        [Fact()]
        public void TestJSONByteSerialization()
        {
            Assert.Equal("212", JsonFunctions.Serialize((byte)212));
            Assert.Equal("0", JsonFunctions.Serialize((byte)0));
            Assert.Equal("45", JsonFunctions.Serialize((byte)45));
            Assert.Equal("59", JsonFunctions.Serialize((byte)59));
            Assert.Equal("11", JsonFunctions.Serialize((byte)11));
            Assert.Equal("2", JsonFunctions.Serialize((byte)2));
            Assert.Equal("40", JsonFunctions.Serialize((byte)40));
            Assert.Equal("13", JsonFunctions.Serialize((byte)13));
        }

        [Fact()]
        public void TestJSONSByteSerialization()
        {
            Assert.Equal("-128", JsonFunctions.Serialize((sbyte)-128));
            Assert.Equal("5", JsonFunctions.Serialize((sbyte)5));
            Assert.Equal("-12", JsonFunctions.Serialize((sbyte)-12));
            Assert.Equal("59", JsonFunctions.Serialize((sbyte)59));
            Assert.Equal("11", JsonFunctions.Serialize((sbyte)11));
            Assert.Equal("-10", JsonFunctions.Serialize((sbyte)-10));
            Assert.Equal("40", JsonFunctions.Serialize((sbyte)40));
            Assert.Equal("13", JsonFunctions.Serialize((sbyte)13));
        }

        [Fact()]
        public void TestJSONCharSerialization()
        {
            Assert.Equal("\" \"", JsonFunctions.Serialize(' '));
            Assert.Equal("\"Z\"", JsonFunctions.Serialize('Z'));
            Assert.Equal("\"\\u0000\"", JsonFunctions.Serialize('\0'));
            Assert.Equal("\"\\f\"", JsonFunctions.Serialize('\f'));
            Assert.Equal("\"0\"", JsonFunctions.Serialize('0'));
            Assert.Equal("\"\\u0018\"", JsonFunctions.Serialize('\x18'));
            Assert.Equal("\"[\"", JsonFunctions.Serialize('['));
            Assert.Equal("\"\\\\\"", JsonFunctions.Serialize('\\'));
        }
    }
}
