﻿// <copyright file="DataReaderPlusDbCommandWrapper.cs" company="Felipe José Angriman">
//   Copyright 2009 Felipe José Angriman 
//
//   Licensed under the Apache License, Version 2.0 (the "License"); 
//   you may not use this file except in compliance with the License. 
//   You may obtain a copy of the License at 
//   
//       http://www.apache.org/licenses/LICENSE-2.0 
// 
//   Unless required by applicable law or agreed to in writing, software
//   distributed under the License is distributed on an "AS IS" BASIS, 
//   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//   See the License for the specific language governing permissions and 
//   limitations under the License. 
// </copyright>
// <author>Felipe José Angriman</author>
// <summary>Defines the DataReaderPlusDbCommandWrapper Class</summary>

namespace Core.Database
{
    using System;
    using System.Data;

    public sealed class DataReaderPlusDbCommandWrapper : IDisposable, IDataReader
    {
        public IDataReader Reader { get; private set; }
        public IDbCommand Command { get; private set; }

        internal DataReaderPlusDbCommandWrapper(IDataReader reader, IDbCommand command)
        {
            this.Reader = reader;
            this.Command = command;
        }

        public void Dispose()
        {
            this.Dispose(true);
            GC.SuppressFinalize(this);
        }

        private void Dispose(bool disposing)
        {
            if (disposing)
            {
            }

            Reader.Close();
            Command.Dispose();
        }
        
        ~DataReaderPlusDbCommandWrapper()
        {
            this.Dispose(false);
        }

        public void Close()
        {
            this.Dispose();
        }

        public int Depth
        {
            get { return this.Reader.Depth; }
        }

        public DataTable GetSchemaTable()
        {
            return this.Reader.GetSchemaTable();
        }

        public bool IsClosed
        {
            get { return this.Reader.IsClosed; }
        }

        public bool NextResult()
        {
            return this.Reader.NextResult();
        }

        public bool Read()
        {
            return this.Reader.Read();
        }

        public int RecordsAffected
        {
            get { return this.Reader.RecordsAffected; }
        }

        public int FieldCount
        {
            get { return this.Reader.FieldCount; }
        }

        public bool GetBoolean(int i)
        {
            return this.Reader.GetBoolean(i);
        }

        public byte GetByte(int i)
        {
            return this.Reader.GetByte(i);
        }

        public long GetBytes(int i, long fieldOffset, byte[] buffer, int bufferoffset, int length)
        {
            return this.Reader.GetBytes(i, fieldOffset, buffer, bufferoffset, length);
        }

        public char GetChar(int i)
        {
            return this.Reader.GetChar(i);
        }

        public long GetChars(int i, long fieldoffset, char[] buffer, int bufferoffset, int length)
        {
            return this.Reader.GetChars(i, fieldoffset, buffer, bufferoffset, length);
        }

        public IDataReader GetData(int i)
        {
            return this.Reader.GetData(i);
        }

        public string GetDataTypeName(int i)
        {
            return this.Reader.GetDataTypeName(i);
        }

        public DateTime GetDateTime(int i)
        {
            return this.Reader.GetDateTime(i);
        }

        public decimal GetDecimal(int i)
        {
            return this.Reader.GetDecimal(i);
        }

        public double GetDouble(int i)
        {
            return this.Reader.GetDouble(i);
        }

        public Type GetFieldType(int i)
        {
            return this.Reader.GetFieldType(i);
        }

        public float GetFloat(int i)
        {
            return this.Reader.GetFloat(i);
        }

        public Guid GetGuid(int i)
        {
            return this.Reader.GetGuid(i);
        }

        public short GetInt16(int i)
        {
            return this.Reader.GetInt16(i);
        }

        public int GetInt32(int i)
        {
            return this.Reader.GetInt32(i);
        }

        public long GetInt64(int i)
        {
            return this.Reader.GetInt64(i);
        }

        public string GetName(int i)
        {
            return this.Reader.GetName(i);
        }

        public int GetOrdinal(string name)
        {
            return this.Reader.GetOrdinal(name);
        }

        public string GetString(int i)
        {
            return this.Reader.GetString(i);
        }

        public object GetValue(int i)
        {
            return this.Reader.GetValue(i);
        }

        public int GetValues(object[] values)
        {
            return this.Reader.GetValues(values);
        }

        public bool IsDBNull(int i)
        {
            return this.Reader.IsDBNull(i);
        }

        public object this[string name]
        {
            get { return this.Reader[name]; }
        }

        public object this[int i]
        {
            get { return this.Reader[i]; }
        }
    }
}
