﻿// <copyright file="DatabaseHelper.cs" company="Felipe José Angriman">
//   Copyright 2009 Felipe José Angriman 
//
//   Licensed under the Apache License, Version 2.0 (the "License"); 
//   you may not use this file except in compliance with the License. 
//   You may obtain a copy of the License at 
//   
//       http://www.apache.org/licenses/LICENSE-2.0 
// 
//   Unless required by applicable law or agreed to in writing, software
//   distributed under the License is distributed on an "AS IS" BASIS, 
//   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//   See the License for the specific language governing permissions and 
//   limitations under the License. 
// </copyright>
// <author>Felipe José Angriman</author>
// <summary>Defines the DatabaseHelper Class</summary>

namespace Core.Database
{
    using System.Data;

    public class DatabaseHelper
    {
        public int DefaultCommandTimeout { get; set; }

        public DatabaseHelper()
        {
            this.DefaultCommandTimeout = 30;
        }

        public DatabaseHelperDataAdapterConstructor DataAdapterConstructor { get; set; }

        public int ExecuteNonQueryCommandText(IDbConnection connection, string text, params IDbDataParameter[] parameters)
        {
            using (IDbCommand command = CreateTextCommand(connection, text, parameters))
            {
                return command.ExecuteNonQuery();
            }
        }

        public object ExecuteScalarCommandText(IDbConnection connection, string text, params IDbDataParameter[] parameters)
        {
            using (IDbCommand command = CreateTextCommand(connection, text, parameters))
            {
                return command.ExecuteScalar();
            }
        }

        public DataSet ExecuteReaderCommandTextAsDataSet(IDbConnection connection, string text, params IDbDataParameter[] parameters)
        {
            DataSet result = new DataSet();
            using (IDbCommand command = CreateTextCommand(connection, text, parameters))
            {
                IDataAdapter adpater = this.DataAdapterConstructor(command);
                adpater.Fill(result);
            }
            return result;
        }

        public DataReaderPlusDbCommandWrapper ExecuteReaderCommandText(IDbConnection connection, string text, params IDbDataParameter[] parameters)
        {
            IDbCommand command = CreateTextCommand(connection, text, parameters);
            return new DataReaderPlusDbCommandWrapper(command.ExecuteReader(), command);
        }

        public int ExecuteNonQueryCommandText(IDbTransaction transaction, string text, params IDbDataParameter[] parameters)
        {
            using (IDbCommand command = CreateTextCommand(transaction, text, parameters))
            {
                return command.ExecuteNonQuery();
            }
        }

        public object ExecuteScalarCommandText(IDbTransaction transaction, string text, params IDbDataParameter[] parameters)
        {
            using (IDbCommand command = CreateTextCommand(transaction, text, parameters))
            {
                return command.ExecuteScalar();
            }
        }

        public DataReaderPlusDbCommandWrapper ExecuteReaderCommandText(IDbTransaction transaction, string text, params IDbDataParameter[] parameters)
        {
            IDbCommand command = CreateTextCommand(transaction, text, parameters);
            return new DataReaderPlusDbCommandWrapper(command.ExecuteReader(), command);
        }

        public DataSet ExecuteReaderCommandTextAsDataSet(IDbTransaction transaction, string text, params IDbDataParameter[] parameters)
        {
            DataSet result = new DataSet();
            using (IDbCommand command = CreateTextCommand(transaction, text, parameters))
            {
                IDataAdapter adpater = this.DataAdapterConstructor(command);
                adpater.Fill(result);
            }
            return result;
        }

        public IDbCommand CreateTextCommand(IDbConnection connection, string text, params IDbDataParameter[] parameters)
        {
            IDbCommand command = connection.CreateCommand();
            command.CommandTimeout = this.DefaultCommandTimeout;
            command.Connection = connection;
            command.CommandText = text;
            
            foreach (IDbDataParameter parameter in parameters)
            {
                command.Parameters.Add(parameter);
            }

            command.CommandType = CommandType.Text;
            return command;
        }

        public IDbCommand CreateTextCommand(IDbTransaction transaction, string text, params IDbDataParameter[] parameters)
        {
            IDbCommand command = transaction.Connection.CreateCommand();
            command.CommandTimeout = this.DefaultCommandTimeout;
            command.Connection = transaction.Connection;
            command.Transaction = transaction;
            command.CommandText = text;
            
            foreach (IDbDataParameter parameter in parameters)
            {
                command.Parameters.Add(parameter);
            }
            
            command.CommandType = CommandType.Text;
            return command;
        }
    }
}
