﻿// <copyright file="SqlServerDatabaseHelper.cs" company="Felipe José Angriman">
//   Copyright 2009 Felipe José Angriman 
//
//   Licensed under the Apache License, Version 2.0 (the "License"); 
//   you may not use this file except in compliance with the License. 
//   You may obtain a copy of the License at 
//   
//       http://www.apache.org/licenses/LICENSE-2.0 
// 
//   Unless required by applicable law or agreed to in writing, software
//   distributed under the License is distributed on an "AS IS" BASIS, 
//   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//   See the License for the specific language governing permissions and 
//   limitations under the License. 
// </copyright>
// <author>Felipe José Angriman</author>
// <summary>Defines the SqlServerDatabaseHelper Class</summary>

using System;
using System.Collections.Generic;
using System.Text;
using System.Data.SqlClient;
using System.Data;

namespace Core.Database
{
    public sealed class SqlServerDatabaseHelper : DatabaseHelper
    {
        public SqlServerDatabaseHelper()
        {
            this.DataAdapterConstructor = delegate(IDbCommand command) { return new SqlDataAdapter((SqlCommand)command); };
        }

        #region "Create[XXXXX]Parameter methods"

        public static SqlParameter CreateBitParameter(string parameterName, object value)
        {
            return CreateBitParameter(parameterName, value, ParameterDirection.Input);
        }

        public static SqlParameter CreateBitParameter(string parameterName, object value, ParameterDirection direction)
        {
            return CreateUnspecifiedSizeParameter(parameterName, value, direction, SqlDbType.Bit);
        }

        public static SqlParameter CreateTinyIntParameter(string parameterName, object value)
        {
            return CreateTinyIntParameter(parameterName, value, ParameterDirection.Input);
        }

        public static SqlParameter CreateTinyIntParameter(string parameterName, object value, ParameterDirection direction)
        {
            return CreateUnspecifiedSizeParameter(parameterName, value, direction, SqlDbType.TinyInt);
        }

        public static SqlParameter CreateSmallIntParameter(string parameterName, object value)
        {
            return CreateSmallIntParameter(parameterName, value, ParameterDirection.Input);
        }

        public static SqlParameter CreateSmallIntParameter(string parameterName, object value, ParameterDirection direction)
        {
            return CreateUnspecifiedSizeParameter(parameterName, value, direction, SqlDbType.SmallInt);
        }

        public static SqlParameter CreateIntParameter(string parameterName, object value)
        {
            return CreateIntParameter(parameterName, value, ParameterDirection.Input);
        }

        public static SqlParameter CreateIntParameter(string parameterName, object value, ParameterDirection direction)
        {
            return CreateUnspecifiedSizeParameter(parameterName, value, direction, SqlDbType.Int);
        }

        public static SqlParameter CreateBigIntParameter(string parameterName, object value)
        {
            return CreateBigIntParameter(parameterName, value, ParameterDirection.Input);
        }

        public static SqlParameter CreateBigIntParameter(string parameterName, object value, ParameterDirection direction)
        {
            return CreateUnspecifiedSizeParameter(parameterName, value, direction, SqlDbType.BigInt);
        }

        public static SqlParameter CreateTimeParameter(string parameterName, object value)
        {
            return CreateTimeParameter(parameterName, value, ParameterDirection.Input);
        }

        public static SqlParameter CreateTimeParameter(string parameterName, object value, ParameterDirection direction)
        {
            return CreateUnspecifiedSizeParameter(parameterName, value, direction, SqlDbType.Time);
        }

        public static SqlParameter CreateDateParameter(string parameterName, object value)
        {
            return CreateDateParameter(parameterName, value, ParameterDirection.Input);
        }

        public static SqlParameter CreateDateParameter(string parameterName, object value, ParameterDirection direction)
        {
            return CreateUnspecifiedSizeParameter(parameterName, value, direction, SqlDbType.Date);
        }

        public static SqlParameter CreateSmallDateTimeParameter(string parameterName, object value)
        {
            return CreateSmallDateTimeParameter(parameterName, value, ParameterDirection.Input);
        }

        public static SqlParameter CreateSmallDateTimeParameter(string parameterName, object value, ParameterDirection direction)
        {
            return CreateUnspecifiedSizeParameter(parameterName, value, direction, SqlDbType.SmallDateTime);
        }

        public static SqlParameter CreateDateTimeParameter(string parameterName, object value)
        {
            return CreateDateTimeParameter(parameterName, value, ParameterDirection.Input);
        }

        public static SqlParameter CreateDateTimeParameter(string parameterName, object value, ParameterDirection direction)
        {
            return CreateUnspecifiedSizeParameter(parameterName, value, direction, SqlDbType.DateTime);
        }

        public static SqlParameter CreateDateTime2Parameter(string parameterName, object value)
        {
            return CreateDateTime2Parameter(parameterName, value, ParameterDirection.Input);
        }

        public static SqlParameter CreateDateTime2Parameter(string parameterName, object value, ParameterDirection direction)
        {
            return CreateUnspecifiedSizeParameter(parameterName, value, direction, SqlDbType.DateTime2);
        }

        public static SqlParameter CreateDateTimeOffsetParameter(string parameterName, object value)
        {
            return CreateDateTimeOffsetParameter(parameterName, value, ParameterDirection.Input);
        }

        public static SqlParameter CreateDateTimeOffsetParameter(string parameterName, object value, ParameterDirection direction)
        {
            return CreateUnspecifiedSizeParameter(parameterName, value, direction, SqlDbType.DateTimeOffset);
        }

        public static SqlParameter CreateSmallMoneyParameter(string parameterName, object value)
        {
            return CreateSmallMoneyParameter(parameterName, value, ParameterDirection.Input);
        }

        public static SqlParameter CreateSmallMoneyParameter(string parameterName, object value, ParameterDirection direction)
        {
            return CreateUnspecifiedSizeParameter(parameterName, value, direction, SqlDbType.SmallMoney);
        }

        public static SqlParameter CreateMoneyParameter(string parameterName, object value)
        {
            return CreateMoneyParameter(parameterName, value, ParameterDirection.Input);
        }

        public static SqlParameter CreateMoneyParameter(string parameterName, object value, ParameterDirection direction)
        {
            return CreateUnspecifiedSizeParameter(parameterName, value, direction, SqlDbType.Money);
        }

        public static SqlParameter CreateCharParameter(string parameterName, object value, short size)
        {
            return CreateCharParameter(parameterName, value, size, ParameterDirection.Input);
        }

        public static SqlParameter CreateCharParameter(string parameterName, object value, short size, ParameterDirection direction)
        {
            return CreateSpecifiedSizeParameter(parameterName, value, size, direction, SqlDbType.NVarChar);
        }

        public static SqlParameter CreateVarCharParameter(string parameterName, object value, short size)
        {
            return CreateVarCharParameter(parameterName, value, size, ParameterDirection.Input);
        }

        public static SqlParameter CreateVarCharParameter(string parameterName, object value, short size, ParameterDirection direction)
        {
            return CreateSpecifiedSizeParameter(parameterName, value, size, direction, SqlDbType.VarChar);
        }

        public static SqlParameter CreateNCharParameter(string parameterName, object value, short size)
        {
            return CreateNCharParameter(parameterName, value, size, ParameterDirection.Input);
        }

        public static SqlParameter CreateNCharParameter(string parameterName, object value, short size, ParameterDirection direction)
        {
            return CreateSpecifiedSizeParameter(parameterName, value, size, direction, SqlDbType.NChar);
        }

        public static SqlParameter CreateNVarCharParameter(string parameterName, object value, short size)
        {
            return CreateNVarCharParameter(parameterName, value, size, ParameterDirection.Input);
        }

        public static SqlParameter CreateNVarCharParameter(string parameterName, object value, short size, ParameterDirection direction)
        {
            return CreateSpecifiedSizeParameter(parameterName, value, size, direction, SqlDbType.NVarChar);
        }

        public static SqlParameter CreateBinaryParameter(string parameterName, object value, short size)
        {
            return CreateBinaryParameter(parameterName, value, size, ParameterDirection.Input);
        }

        public static SqlParameter CreateBinaryParameter(string parameterName, object value, short size, ParameterDirection direction)
        {
            return CreateSpecifiedSizeParameter(parameterName, value, size, direction, SqlDbType.NChar);
        }

        public static SqlParameter CreateVarBinaryParameter(string parameterName, object value, short size)
        {
            return CreateVarBinaryParameter(parameterName, value, size, ParameterDirection.Input);
        }

        public static SqlParameter CreateVarBinaryParameter(string parameterName, object value, short size, ParameterDirection direction)
        {
            return CreateSpecifiedSizeParameter(parameterName, value, size, direction, SqlDbType.NVarChar);
        }

        public static SqlParameter CreateTextParameter(string parameterName, object value)
        {
            return CreateTextParameter(parameterName, value, ParameterDirection.Input);
        }

        public static SqlParameter CreateTextParameter(string parameterName, object value, ParameterDirection parameterDirection)
        {
            return CreateUnspecifiedSizeParameter(parameterName, value, parameterDirection, SqlDbType.Text);
        }

        public static SqlParameter CreateNTextParameter(string parameterName, object value)
        {
            return CreateNTextParameter(parameterName, value, ParameterDirection.Input);
        }

        public static SqlParameter CreateNTextParameter(string parameterName, object value, ParameterDirection parameterDirection)
        {
            return CreateUnspecifiedSizeParameter(parameterName, value, parameterDirection, SqlDbType.NText);
        }

        public static SqlParameter CreateImageParameter(string parameterName, object value)
        {
            return CreateImageParameter(parameterName, value, ParameterDirection.Input);
        }

        public static SqlParameter CreateImageParameter(string parameterName, object value, ParameterDirection parameterDirection)
        {
            return CreateUnspecifiedSizeParameter(parameterName, value, parameterDirection, SqlDbType.Image);
        }

        public static SqlParameter CreateXmlParameter(string parameterName, object value)
        {
            return CreateXmlParameter(parameterName, value, ParameterDirection.Input);
        }

        public static SqlParameter CreateXmlParameter(string parameterName, object value, ParameterDirection parameterDirection)
        {
            return CreateUnspecifiedSizeParameter(parameterName, value, parameterDirection, SqlDbType.Xml);
        }

        public static SqlParameter CreateUniqueIdentifierParameter(string parameterName, object value)
        {
            return CreateUniqueIdentifierParameter(parameterName, value, ParameterDirection.Input);
        }

        public static SqlParameter CreateUniqueIdentifierParameter(string parameterName, object value, ParameterDirection parameterDirection)
        {
            return CreateUnspecifiedSizeParameter(parameterName, value, parameterDirection, SqlDbType.UniqueIdentifier);
        }

        public static SqlParameter CreateTimestampParameter(string parameterName, object value)
        {
            return CreateTimestampParameter(parameterName, value, ParameterDirection.Input);
        }

        public static SqlParameter CreateTimestampParameter(string parameterName, object value, ParameterDirection parameterDirection)
        {
            return CreateUnspecifiedSizeParameter(parameterName, value, parameterDirection, SqlDbType.Timestamp);
        }

        public static SqlParameter CreateFloatParameter(string parameterName, object value)
        {
            return CreateFloatParameter(parameterName, value, ParameterDirection.Input);
        }

        public static SqlParameter CreateFloatParameter(string parameterName, object value, ParameterDirection parameterDirection)
        {
            return CreateUnspecifiedSizeParameter(parameterName, value, parameterDirection, SqlDbType.Float);
        }

        public static SqlParameter CreateRealParameter(string parameterName, object value)
        {
            return CreateRealParameter(parameterName, value, ParameterDirection.Input);
        }

        public static SqlParameter CreateRealParameter(string parameterName, object value, ParameterDirection parameterDirection)
        {
            return CreateUnspecifiedSizeParameter(parameterName, value, parameterDirection, SqlDbType.Real);
        }

        public static SqlParameter CreateDecimalParameter(string parameterName, object value, byte precision, byte scale)
        {
            return CreateDecimalParameter(parameterName, value, precision, scale, ParameterDirection.Input);
        }

        public static SqlParameter CreateDecimalParameter(string parameterName, object value, byte precision, byte scale, ParameterDirection direction)
        {
            SqlParameter parameter = new SqlParameter();
            parameter.ParameterName = GetParameterName(parameterName);
            parameter.Value = value ?? DBNull.Value;
            parameter.Direction = direction;
            parameter.Precision = precision;
            parameter.Scale = scale;
            parameter.SqlDbType = SqlDbType.Decimal;
            return parameter;
        }

        public static SqlParameter CreateSpecifiedSizeParameter(string parameterName, object value, short size, ParameterDirection direction, SqlDbType sqlDbType)
        {
            SqlParameter parameter = new SqlParameter();
            parameter.ParameterName = GetParameterName(parameterName);
            parameter.Value = value ?? DBNull.Value;
            parameter.Size = size;
            parameter.Direction = direction;
            parameter.SqlDbType = sqlDbType;
            return parameter;
        }

        private static SqlParameter CreateUnspecifiedSizeParameter(string parameterName, object value, ParameterDirection direction, SqlDbType sqlDbType)
        {
            SqlParameter parameter = new SqlParameter();
            parameter.ParameterName = GetParameterName(parameterName);
            parameter.Value = value ?? DBNull.Value;
            parameter.Direction = direction;
            parameter.SqlDbType = sqlDbType;
            return parameter;
        }
        
        #endregion

        private static string GetParameterName(string parameterName)
        {
            if (!parameterName.StartsWith("@", StringComparison.InvariantCulture))
            {
                return "@" + parameterName;
            }
            else
            {
                return parameterName;
            }
        }
    }
}
