﻿// <copyright file="JsonParser.cs" company="Felipe José Angriman">
//   Copyright 2009 Felipe José Angriman 
//
//   Licensed under the Apache License, Version 2.0 (the "License"); 
//   you may not use this file except in compliance with the License. 
//   You may obtain a copy of the License at 
//   
//       http://www.apache.org/licenses/LICENSE-2.0 
// 
//   Unless required by applicable law or agreed to in writing, software
//   distributed under the License is distributed on an "AS IS" BASIS, 
//   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//   See the License for the specific language governing permissions and 
//   limitations under the License. 
// </copyright>
// <author>Felipe José Angriman</author>
// <summary>Defines the JsonParser Class</summary>

namespace Core.Json
{
    using System;
    using System.Collections.Generic;
    using System.Text;

    public class JsonParser
    {
        private JsonReader reader;
        private JsonToken token;
        private JsonToken nextToken;

        public JsonParser(JsonReader reader)
        {
            if (reader == null)
            {
                throw new ArgumentNullException("reader");
            }

            this.reader = reader;
            this.token = new JsonToken();
            this.nextToken = new JsonToken();
        }
        
        private void Object()
        {
            this.Match(JsonTokenId.OpeningBraces);
            this.PropertyList();
            this.Match(JsonTokenId.ClosingBraces);
        }

        private void Array()
        {
            this.Match(JsonTokenId.OpeningBrackets);
            this.ValueList();
            this.Match(JsonTokenId.ClosingBrackets);
        }

        private void ValueList()
        {
            switch (this.nextToken.TokenId)
            {
                case JsonTokenId.NumberLiteral:
                case JsonTokenId.StringLiteral:
                case JsonTokenId.TrueLiteral:
                case JsonTokenId.FalseLiteral:
                case JsonTokenId.NullLiteral:
                case JsonTokenId.OpeningBraces:
                case JsonTokenId.OpeningBrackets:
                    this.Value();
                    while (this.nextToken.TokenId == JsonTokenId.Comma)
                    {
                        this.Match(JsonTokenId.Comma);
                        this.Value();
                    }
                    break;
                case JsonTokenId.ClosingBrackets:
                    break;
                default:
                    break;
            }
        }

        private void PropertyList()
        {
            switch (this.nextToken.TokenId)
            {
                case JsonTokenId.StringLiteral:
                    this.Property();
                    while (this.nextToken.TokenId == JsonTokenId.Comma)
                    {
                        this.Match(JsonTokenId.Comma);
                        this.Property();
                    }

                    break;
                case JsonTokenId.ClosingBraces:
                    break;
                default:
                    break;
            }
        }

        private void Value()
        {
            switch (this.nextToken.TokenId)
            {
                case JsonTokenId.NumberLiteral:
                    this.Match(JsonTokenId.NumberLiteral);
                    break;
                case JsonTokenId.StringLiteral:
                    this.Match(JsonTokenId.StringLiteral);
                    break;
                case JsonTokenId.TrueLiteral:
                    this.Match(JsonTokenId.TrueLiteral);
                    break;
                case JsonTokenId.FalseLiteral:
                    this.Match(JsonTokenId.FalseLiteral);
                    break;
                case JsonTokenId.NullLiteral:
                    this.Match(JsonTokenId.NullLiteral);
                    break;
                case JsonTokenId.OpeningBraces:
                    this.Object();
                    break;
                case JsonTokenId.OpeningBrackets:
                    this.Array();
                    break;
                default:
                    break;
            }
        }

        private void Match(JsonTokenId jsonTokenId)
        {
            throw new NotImplementedException();
        }

        private void Property()
        {
            this.Match(JsonTokenId.StringLiteral);
            this.Match(JsonTokenId.Colon);
            this.Value();
        }
    }
}
