﻿// <copyright file="JsonTokenId.cs" company="Felipe José Angriman">
//   Copyright 2009 Felipe José Angriman 
//
//   Licensed under the Apache License, Version 2.0 (the "License"); 
//   you may not use this file except in compliance with the License. 
//   You may obtain a copy of the License at 
//   
//       http://www.apache.org/licenses/LICENSE-2.0 
// 
//   Unless required by applicable law or agreed to in writing, software
//   distributed under the License is distributed on an "AS IS" BASIS, 
//   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//   See the License for the specific language governing permissions and 
//   limitations under the License. 
// </copyright>
// <author>Felipe José Angriman</author>
// <summary>Defines the JsonTokenId Class</summary>

namespace Core.Json
{

    /// <summary>
    /// The tokens types found on json text.
    /// </summary>
    public enum JsonTokenId
    {
        /// <summary>
        /// A Json number.
        /// </summary>
        NumberLiteral,
        /// <summary>
        /// A Json string.
        /// </summary>
        StringLiteral,
        /// <summary>
        /// A true literal.
        /// </summary>
        TrueLiteral,
        /// <summary>
        /// A false literal.
        /// </summary>
        FalseLiteral,
        /// <summary>
        /// A null literal.
        /// </summary>
        NullLiteral,
        /// <summary>
        /// A comma.
        /// </summary>
        Comma,
        /// <summary>
        /// A colon.
        /// </summary>
        Colon,
        OpeningBraces,
        ClosingBraces,
        OpeningBrackets,
        ClosingBrackets,
        /// <summary>
        /// The end of input.
        /// </summary>
        EndOfInput,
        /// <summary>
        /// An invalid token.
        /// </summary>
        Error
    }
}
