﻿// <copyright file="DBNullSerializer.cs" company="Felipe José Angriman">
//   Copyright 2009 Felipe José Angriman 
//
//   Licensed under the Apache License, Version 2.0 (the "License"); 
//   you may not use this file except in compliance with the License. 
//   You may obtain a copy of the License at 
//   
//       http://www.apache.org/licenses/LICENSE-2.0 
// 
//   Unless required by applicable law or agreed to in writing, software
//   distributed under the License is distributed on an "AS IS" BASIS, 
//   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//   See the License for the specific language governing permissions and 
//   limitations under the License. 
// </copyright>
// <author>Felipe José Angriman</author>
// <summary>Defines the DBNullSerializer Class</summary>

namespace Core.Json.Serializers
{
    using System;
    using Core.Reflection.TypeExpressions;
    using TE = Core.Reflection.TypeExpressions.TypeExpression;

    public sealed class DBNullSerializer : ITypeSerializer
    {
        private TypeExpression typeMatcher = new TypeEqualsExpression(new TypeVariableReferenceExpression("type"), new TypeConstantReferenceExpression(typeof(DBNull)));

        public TypeExpression TypeExpression
        {
            get
            {
                return this.typeMatcher;
            }
        }

        public void Serialize(object source, JsonWriter writer)
        {
            if (source == null)
            {
                throw new ArgumentNullException("source");
            }

            if (writer == null)
            {
                throw new ArgumentNullException("writer");
            }

            writer.Serialize(null);
        }
    }
}
