﻿// <copyright file="DoubleSerializer.cs" company="Felipe José Angriman">
//   Copyright 2009 Felipe José Angriman 
//
//   Licensed under the Apache License, Version 2.0 (the "License"); 
//   you may not use this file except in compliance with the License. 
//   You may obtain a copy of the License at 
//   
//       http://www.apache.org/licenses/LICENSE-2.0 
// 
//   Unless required by applicable law or agreed to in writing, software
//   distributed under the License is distributed on an "AS IS" BASIS, 
//   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//   See the License for the specific language governing permissions and 
//   limitations under the License. 
// </copyright>
// <author>Felipe José Angriman</author>
// <summary>Defines the DoubleSerializer Class</summary>

namespace Core.Json.Serializers
{
    using System;
    using System.Globalization;
    using Core.Reflection.TypeExpressions;
    using TE = Core.Reflection.TypeExpressions.TypeEqualsExpression;

    public sealed class DoubleSerializer : ITypeSerializer
    {
        private static readonly TypeExpression typeExpression = TE.Equals(TE.VariableReference("type"), TE.ConstantReference(typeof(double)));

        public TypeExpression TypeExpression
        {
            get
            {
                return typeExpression;
            }
        }

        public void Serialize(object source, JsonWriter writer)
        {
            if (source == null)
            {
                throw new ArgumentNullException("source");
            }

            if (writer == null)
            {
                throw new ArgumentNullException("writer");
            }

            double f = (double)source;

            if (double.IsNaN(f))
            {
                writer.WriteVerbatimString("NaN");
            }

            if (double.IsPositiveInfinity(f))
            {
                writer.WriteVerbatimString("+Inf");
            }

            if (double.IsNegativeInfinity(f))
            {
                writer.WriteVerbatimString("-Inf");
            }

            writer.WriteVerbatim(f.ToString("r", CultureInfo.InvariantCulture));
        }

    }
}
