﻿// <copyright file="IEnumerableSerializer.cs" company="Felipe José Angriman">
//   Copyright 2009 Felipe José Angriman 
//
//   Licensed under the Apache License, Version 2.0 (the "License"); 
//   you may not use this file except in compliance with the License. 
//   You may obtain a copy of the License at 
//   
//       http://www.apache.org/licenses/LICENSE-2.0 
// 
//   Unless required by applicable law or agreed to in writing, software
//   distributed under the License is distributed on an "AS IS" BASIS, 
//   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//   See the License for the specific language governing permissions and 
//   limitations under the License. 
// </copyright>
// <author>Felipe José Angriman</author>
// <summary>Defines the IntegerTypeSerializer Class</summary>

namespace Core.Json.Serializers
{
    using System.Globalization;
    using Core.Reflection.TypeExpressions;
    using TE = Core.Reflection.TypeExpressions.TypeExpression;
    using System;

    public sealed class IntegerTypeSerializer : ITypeSerializer
    {
        private static readonly TypeExpression typeExpression =
            TE.Or(
                TE.Or(
                    TE.Or(
                        TE.Or(
                            TE.Or(
                                TE.Or(
                                    TE.Or(
                                        TE.Equals(TE.VariableReference("type"), TE.ConstantReference(typeof(byte))),
                                        TE.Equals(TE.VariableReference("type"), TE.ConstantReference(typeof(sbyte)))),
                                    TE.Equals(TE.VariableReference("type"), TE.ConstantReference(typeof(short)))),
                                TE.Equals(TE.VariableReference("type"), TE.ConstantReference(typeof(ushort)))),
                            TE.Equals(TE.VariableReference("type"), TE.ConstantReference(typeof(int)))),
                        TE.Equals(TE.VariableReference("type"), TE.ConstantReference(typeof(uint)))),
                    TE.Equals(TE.VariableReference("type"), TE.ConstantReference(typeof(long)))),
                TE.Equals(TE.VariableReference("type"), TE.ConstantReference(typeof(ulong))));

        public TypeExpression TypeExpression
        {
            get
            {
                return typeExpression;
            }
        }

        public void Serialize(object source, JsonWriter writer)
        {
            if (source == null)
            {
                throw new ArgumentNullException("source");
            }

            if (writer == null)
            {
                throw new ArgumentNullException("writer");
            }

            writer.WriteVerbatim(string.Format(CultureInfo.InvariantCulture, "{0}", source));
        }

    }
}
