﻿// <copyright file="IReadablePropertyCursor.cs" company="Felipe José Angriman">
//   Copyright 2009 Felipe José Angriman 
//
//   Licensed under the Apache License, Version 2.0 (the "License"); 
//   you may not use this file except in compliance with the License. 
//   You may obtain a copy of the License at 
//   
//       http://www.apache.org/licenses/LICENSE-2.0 
// 
//   Unless required by applicable law or agreed to in writing, software
//   distributed under the License is distributed on an "AS IS" BASIS, 
//   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//   See the License for the specific language governing permissions and 
//   limitations under the License. 
// </copyright>
// <author>Felipe José Angriman</author>
// <summary>Defines the IReadablePropertyCursor Class</summary>

namespace Core.Reflection.Readers
{
    using System;

    /// <summary>
    /// A cursor over the properties of an object.
    /// </summary>
    public interface IReadablePropertyCursor 
    {
        /// <summary>
        /// Gets the object on which the cursor operates.
        /// </summary>
        object Object
        {
            get;
        }
        
        /// <summary>
        /// Advances the cursor to the next property.
        /// </summary>
        /// <returns>true when the cursor is moved to a new property, false when the cursor has gone past the last property.</returns>
        bool MoveToNextProperty();

        /// <summary>
        /// Gets the type of the current property.
        /// </summary>
        Type CurrentPropertyType 
        { 
            get; 
        }

        /// <summary>
        /// Gets the name of the current property.
        /// </summary>
        string CurrentPropertyName 
        { 
            get; 
        }

        /// <summary>
        /// Reads the value of the current property.
        /// </summary>
        object ReadCurrentProperty();
    }
}
