﻿// <copyright file="ObjectReadableField.cs" company="Felipe José Angriman">
//   Copyright 2009 Felipe José Angriman 
//
//   Licensed under the Apache License, Version 2.0 (the "License"); 
//   you may not use this file except in compliance with the License. 
//   You may obtain a copy of the License at 
//   
//       http://www.apache.org/licenses/LICENSE-2.0 
// 
//   Unless required by applicable law or agreed to in writing, software
//   distributed under the License is distributed on an "AS IS" BASIS, 
//   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//   See the License for the specific language governing permissions and 
//   limitations under the License. 
// </copyright>
// <author>Felipe José Angriman</author>
// <summary>Defines the ObjectReadableField Class</summary>

namespace Core.Reflection.Readers
{
    using System;
    using System.Reflection;

    public sealed class ObjectReadableField : IReadableProperty
    {
        private FieldInfo fieldInfo;

        internal ObjectReadableField(object @object, FieldInfo fieldInfo)
        {
            this.Object = @object;
            this.fieldInfo = fieldInfo;
        }

        public object Object
        {
            get;
            private set;
        }

        public Type PropertyType
        {
            get
            {
                return this.fieldInfo.FieldType;
            }
        }

        public string PropertyName
        {
            get 
            { 
                return fieldInfo.Name; 
            }
        }

        public object Read()
        {
            return this.fieldInfo.GetValue(this.Object);
        }
    }
}
