﻿// <copyright file="ObjectReadableProperty.cs" company="Felipe José Angriman">
//   Copyright 2009 Felipe José Angriman 
//
//   Licensed under the Apache License, Version 2.0 (the "License"); 
//   you may not use this file except in compliance with the License. 
//   You may obtain a copy of the License at 
//   
//       http://www.apache.org/licenses/LICENSE-2.0 
// 
//   Unless required by applicable law or agreed to in writing, software
//   distributed under the License is distributed on an "AS IS" BASIS, 
//   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//   See the License for the specific language governing permissions and 
//   limitations under the License. 
// </copyright>
// <author>Felipe José Angriman</author>
// <summary>Defines the ObjectReadableProperty Class</summary>

namespace Core.Reflection.Readers
{
    using System;
    using System.Reflection;

    /// <summary>
    /// Represents a readable property of an object.
    /// </summary>
    public sealed class ObjectReadableProperty : IReadableProperty
    {
        private PropertyInfo properyInfo;

        internal ObjectReadableProperty(object obj, PropertyInfo propertyInfo)
        {
            this.Object = obj;
            this.properyInfo = propertyInfo;
        }

        public object Object
        {
            get;
            private set;
        }


        public Type PropertyType
        {
            get 
            {
                return this.properyInfo.PropertyType;
            }
        }

        /// <summary>
        /// Gets the name of the property
        /// </summary>
        public string PropertyName
        {
            get 
            { 
                return properyInfo.Name; 
            }
        }

        /// <summary>
        /// Gets the value of the property.
        /// </summary>
        public object Read()
        {
            return this.properyInfo.GetGetMethod().Invoke(this.Object, null);
        }

    }
}
