﻿// <copyright file="ReaderManager.cs" company="Felipe José Angriman">
//   Copyright 2009 Felipe José Angriman 
//
//   Licensed under the Apache License, Version 2.0 (the "License"); 
//   you may not use this file except in compliance with the License. 
//   You may obtain a copy of the License at 
//   
//       http://www.apache.org/licenses/LICENSE-2.0 
// 
//   Unless required by applicable law or agreed to in writing, software
//   distributed under the License is distributed on an "AS IS" BASIS, 
//   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//   See the License for the specific language governing permissions and 
//   limitations under the License. 
// </copyright>
// <author>Felipe José Angriman</author>
// <summary>Defines the ReaderManager Class</summary>

namespace Core.Reflection.Readers
{
    using System;
    using System.Collections.Generic;
    using Core.Reflection.TypeExpressions;
    using Core.Json.Serializers;

    public sealed class ReaderManager
    {
        private readonly List<IReader> readers = new List<IReader>();

        public IList<IReader> Readers 
        {
            get
            {
                return this.readers;
            }
        }

        public IReader GetReader(object source)
        {
            TypeExpressionEvaluationVisitor visitor = new TypeExpressionEvaluationVisitor();
            Dictionary<string, Type> dic = new Dictionary<string, Type>();
            dic.Add("type", source.GetType());

            foreach (IReader reader in this.readers)
            {
                if (visitor.Evaluate(reader.TypeExpression, dic))
                {
                    return reader;
                }
            }

            return null;
        }
    }
}
