﻿// <copyright file="IEnumerableHelperTests.cs" company="Felipe José Angriman">
//   Copyright 2009 Felipe José Angriman 
//
//   Licensed under the Apache License, Version 2.0 (the "License"); 
//   you may not use this file except in compliance with the License. 
//   You may obtain a copy of the License at 
//   
//       http://www.apache.org/licenses/LICENSE-2.0 
// 
//   Unless required by applicable law or agreed to in writing, software
//   distributed under the License is distributed on an "AS IS" BASIS, 
//   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//   See the License for the specific language governing permissions and 
//   limitations under the License. 
// </copyright>
// <author>Felipe José Angriman</author>
// <summary>Defines the IEnumerableHelperTests Class</summary>

namespace Core.Tests.Collections
{
    using System;
    using Core.Collections;
    using Xunit;

    public sealed class IEnumerableHelperTests
    {
        [Fact]
        public void ContainsThrowsArgumentNullExceptionWhenEnumerableArgumentIsNullTest()
        {
            Assert.Throws<ArgumentNullException>(delegate()
            {
                IEnumerableHelper.Contains<object>(null, null);
            });
        }

        [Fact]
        public void ContainsReturnsTrueWhenItemIsPresent()
        {
            int[] enumerable = new int[] { 1, 6, 9, 33 };
            Assert.True(IEnumerableHelper.Contains(enumerable, 9));
        }

        [Fact]
        public void ContainsReturnsFalseWhenItemIsNotPresent()
        {
            int[] enumerable = new int[] { 950, 0, 2, -568 };
            Assert.False(IEnumerableHelper.Contains(enumerable, 1));
        }

        [Fact]
        public void CountThrowsArgumentNullExceptionWhenEnumerableArgumentIsNullTest()
        {
            Assert.Throws<ArgumentNullException>(delegate()
            {
                IEnumerableHelper.Count<object>(null);
            });
        }

        [Fact]
        public void CountReturnsZeroWhenIEnumerableIsEmptyTest()
        {
            Assert.Equal(0, IEnumerableHelper.Count(new object[0]));
        }

        [Fact]
        public void CountReturnsNonZeroWhenIEnumerableIsNonEmptyTest()
        {
            Assert.NotEqual(0, IEnumerableHelper.Count(new object[] { "2", 1, new object() }));
        }

        [Fact]
        public void EnumerateDifferenceWithThrowsArgumentNullExceptionWhenFirstEnumerableArgumentIsNullTest()
        {
            Assert.Throws(typeof(ArgumentNullException), delegate()
            {
                IEnumerableHelper.EnumerateDifferenceWith(null, new object[0]);
            });
        }
        
        [Fact]
        public void EnumerateDifferenceWithThrowsArgumentNullExceptionWhenSecondEnumerableArgumentIsNullTest()
        {
            Assert.Throws(typeof(ArgumentNullException), delegate()
            {
                IEnumerableHelper.EnumerateDifferenceWith(new object[0], null);
            });
        }

        [Fact]
        public void EnumerateDifferenceWithReturnEmptyIEnumerableOnDisjointIEnumerablesTest()
        {
            Assert.True(IEnumerableHelper.SetEqual(new int[0], IEnumerableHelper.EnumerateDifferenceWith(new int[] { 0, 1, 2 }, new int[] { 3, 4, 5 })));
        }

        [Fact]
        public void EnumerateIntersectionWithThrowsArgumentNullExceptionWhenFirstEnumerableArgumentIsNullTest()
        {
            Assert.Throws(typeof(ArgumentNullException), delegate()
            {
                IEnumerableHelper.EnumerateIntersectionWith(null, new object[0]);
            });
        }

        [Fact]
        public void EnumerateIntersectionWithThrowsArgumentNullExceptionWhenSecondEnumerableArgumentIsNullTest()
        {
            Assert.Throws(typeof(ArgumentNullException), delegate()
            {
                IEnumerableHelper.EnumerateIntersectionWith(new object[0], null);
            });
        }
    }
}
