﻿// <copyright file="DataClassPattern.cs" company="Felipe José Angriman">
//   Copyright 2009 Felipe José Angriman 
//
//   Licensed under the Apache License, Version 2.0 (the "License"); 
//   you may not use this file except in compliance with the License. 
//   You may obtain a copy of the License at 
//   
//       http://www.apache.org/licenses/LICENSE-2.0 
// 
//   Unless required by applicable law or agreed to in writing, software
//   distributed under the License is distributed on an "AS IS" BASIS, 
//   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//   See the License for the specific language governing permissions and 
//   limitations under the License. 
// </copyright>
// <author>Felipe José Angriman</author>
// <summary>Defines the DataClassPattern Class</summary>

namespace Core.CodeDom.Pattern
{
    using System;
    using System.CodeDom;

    /// <summary>
    /// A pattern for building simple data classes.
    /// </summary>
    public sealed class DataClassPattern : IPattern<DataClassPatternInput, DataClassPatternOutput>
    {
        /// <summary>
        /// Initializes a new instance of the DataClassPattern class.
        /// </summary>
        public DataClassPattern()
        {
        }

        /// <summary>
        /// Gets or sets the context of this pattern.
        /// </summary>
        public IPatternContext Context
        {
            get;
            set;
        }

        /// <summary>
        /// Produces the data class.
        /// </summary>
        /// <param name="input">The input data for the pattern.</param>
        /// <returns>The data class contained in a DataClassPatternOutput instance.</returns>
        public DataClassPatternOutput ProduceOutput(DataClassPatternInput input)
        {
            if (input == null)
            {
                throw new ArgumentNullException("input");
            }

                
            DataClassPatternOutput output = new DataClassPatternOutput();
            CodeTypeDeclaration dataClass = new CodeTypeDeclaration();
            ReadWritePropertyPattern rwpp = new ReadWritePropertyPattern();

            dataClass.Name = input.Name;
            dataClass.TypeAttributes = input.Attributes;
            dataClass.Members.Add(CodeTypeMemberBuilder.CreatePublicParameterlessConstructor());

            foreach (var propertyData in input.Properties)
            {
                ReadWritePropertyPatternOutput propertyOutput = rwpp.ProduceOutput(new ReadWritePropertyPatternInput
                {
                    PropertyType = propertyData.PropertyType,
                    PropertyName = propertyData.PropertyName
                });
                dataClass.Members.Add(propertyOutput.Field);
                dataClass.Members.Add(propertyOutput.Property);
            }

            output.Class = dataClass;

            return output;
        }
    }
}
