﻿// <copyright file="IPattern.cs" company="Felipe José Angriman">
//   Copyright 2009 Felipe José Angriman 
//
//   Licensed under the Apache License, Version 2.0 (the "License"); 
//   you may not use this file except in compliance with the License. 
//   You may obtain a copy of the License at 
//   
//       http://www.apache.org/licenses/LICENSE-2.0 
// 
//   Unless required by applicable law or agreed to in writing, software
//   distributed under the License is distributed on an "AS IS" BASIS, 
//   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//   See the License for the specific language governing permissions and 
//   limitations under the License. 
// </copyright>
// <author>Felipe José Angriman</author>
// <summary>Defines the IPattern Interface</summary>

namespace Core.CodeDom.Pattern
{
    /// <summary>
    /// A pattern for generating code.
    /// </summary>
    /// <typeparam name="TPatternInput">The input for the pattern.</typeparam>
    /// <typeparam name="TPatternOutput">The output for the pattern.</typeparam>
    public interface IPattern<TPatternInput, TPatternOutput>
    {
        /// <summary>
        /// Gets or sets the context assigned to this pattern.
        /// </summary>
        IPatternContext Context
        {
            get;
            set;
        }

        /// <summary>
        /// Builds the code artifacts.
        /// </summary>
        /// <param name="input"></param>
        /// <returns></returns>
        TPatternOutput ProduceOutput(TPatternInput input);
    }
}
