﻿// <copyright file="IPatternContext.cs" company="Felipe José Angriman">
//   Copyright 2009 Felipe José Angriman 
//
//   Licensed under the Apache License, Version 2.0 (the "License"); 
//   you may not use this file except in compliance with the License. 
//   You may obtain a copy of the License at 
//   
//       http://www.apache.org/licenses/LICENSE-2.0 
// 
//   Unless required by applicable law or agreed to in writing, software
//   distributed under the License is distributed on an "AS IS" BASIS, 
//   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//   See the License for the specific language governing permissions and 
//   limitations under the License. 
// </copyright>
// <author>Felipe José Angriman</author>
// <summary>Defines the IPatternContext Interface</summary>

namespace Core.CodeDom.Pattern
{
    using System.CodeDom;
    using System.CodeDom.Compiler;

    /// <summary>
    /// The Context in which patterns runs
    /// </summary>
    public interface IPatternContext
    {
        /// <summary>
        /// Gets the CodeDomProvider object that will be used for code generation.
        /// </summary>
        CodeDomProvider Provider
        {
            get;
        }

        /// <summary>
        /// Gets the CodeNamespace object that contains auxiliary types that are created during CodeDom tree generation
        /// </summary>
        CodeNamespace AuxiliaryTypesNamespace
        {
            get;
        }

        /// <summary>
        /// Gets the CodeTypeDeclaration object that contains auxiliary types that are created during CodeDom tree generation
        /// </summary>
        CodeTypeDeclaration AuxiliaryStaticMethodsType
        {
            get;
        }
    }
}
