﻿// <copyright file="ReadWritePropertyPattern.cs" company="Felipe José Angriman">
//   Copyright 2009 Felipe José Angriman 
//
//   Licensed under the Apache License, Version 2.0 (the "License"); 
//   you may not use this file except in compliance with the License. 
//   You may obtain a copy of the License at 
//   
//       http://www.apache.org/licenses/LICENSE-2.0 
// 
//   Unless required by applicable law or agreed to in writing, software
//   distributed under the License is distributed on an "AS IS" BASIS, 
//   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//   See the License for the specific language governing permissions and 
//   limitations under the License. 
// </copyright>
// <author>Felipe José Angriman</author>
// <summary>Defines the ReadWritePropertyPattern Class</summary>

namespace Core.CodeDom.Pattern
{
    using System.CodeDom;

    /// <summary>
    /// A Pattern for building simple read-write properties.
    /// </summary>
    public sealed class ReadWritePropertyPattern : IPattern<ReadWritePropertyPatternInput, ReadWritePropertyPatternOutput>
    {
        /// <summary>
        /// Initializes a new instance of the ReadWritePropertyPattern class.
        /// </summary>
        public ReadWritePropertyPattern()
        {
        }

        public IPatternContext Context
        {
            get;
            set;
        }

        public ReadWritePropertyPatternOutput ProduceOutput(ReadWritePropertyPatternInput input)
        {
            ReadWritePropertyPatternOutput output = new ReadWritePropertyPatternOutput();
            CodeMemberField field = new CodeMemberField();
            CodeMemberProperty property = new CodeMemberProperty();

            field.Type = input.PropertyType;
            property.Name = "_" + input.PropertyName;
            property.Type = input.PropertyType;
            property.Name = input.PropertyName;
            property.Attributes = MemberAttributes.Public | MemberAttributes.Final;
            property.HasGet = true;
            property.HasSet = true;
            property.GetStatements.Add(new CodeMethodReturnStatement(new CodeFieldReferenceExpression(new CodeThisReferenceExpression() , field.Name)));
            property.SetStatements.Add(new CodeAssignStatement(new CodeFieldReferenceExpression(new CodeThisReferenceExpression() , field.Name), new CodePropertySetValueReferenceExpression()));

            output.Field = field;
            output.Property = property;
            
            return output;
        }
        
    }
}
