﻿// <copyright file="VisitorPattern.cs" company="Felipe José Angriman">
//   Copyright 2009 Felipe José Angriman 
//
//   Licensed under the Apache License, Version 2.0 (the "License"); 
//   you may not use this file except in compliance with the License. 
//   You may obtain a copy of the License at 
//   
//       http://www.apache.org/licenses/LICENSE-2.0 
// 
//   Unless required by applicable law or agreed to in writing, software
//   distributed under the License is distributed on an "AS IS" BASIS, 
//   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//   See the License for the specific language governing permissions and 
//   limitations under the License. 
// </copyright>
// <author>Felipe José Angriman</author>
// <summary>Defines the VisitorPattern Class</summary>

namespace Core.CodeDom.Pattern
{
    using System;
    using System.CodeDom;
    
    public sealed class VisitorPattern : IPattern<VisitorPatternInput, VisitorPatternOutput>
    {

        public IPatternContext Context 
        { 
            get; 
            set; 
        }

        public VisitorPatternOutput ProduceOutput(VisitorPatternInput input)
        {
            VisitorPatternOutput output = new VisitorPatternOutput();
            CodeTypeDeclaration rootClass = new CodeTypeDeclaration();
            CodeTypeDeclaration visitorClass = new CodeTypeDeclaration();
            rootClass.Name = input.RootClassName;
            rootClass.Attributes = MemberAttributes.Public | MemberAttributes.Abstract;
            
            CodeMemberMethod acceptMethod = GetAcceptMethod(visitorClass.Name);
            CodeMemberMethod acceptedMethod = GetAcceptedMethod(visitorClass.Name, acceptMethod.Name);

            foreach (var @class in input.Classes)
            {
                visitorClass.Members.Add(GetVisitMethod(@class.Name));
            }

            rootClass.Members.Add(acceptMethod);
            rootClass.Members.Add(acceptedMethod);
            
            return output;
        }

        private static CodeMemberMethod GetVisitMethod(string className)
        {
            CodeMemberMethod result = new CodeMemberMethod();
            result.Attributes = MemberAttributes.Public | MemberAttributes.Abstract;
            result.Parameters.Add(new CodeParameterDeclarationExpression { Name = "node", Direction = FieldDirection.In, Type = new CodeTypeReference(className) });
            return result;
        }

        private static CodeMemberMethod GetAcceptMethod(string visitorClassName)
        {
            CodeMemberMethod result = new CodeMemberMethod();
            CodeParameterDeclarationExpression visitorParameter = new CodeParameterDeclarationExpression();
            visitorParameter.Name = "visitor";
            visitorParameter.Type = new CodeTypeReference(visitorClassName);
            result.Attributes = MemberAttributes.Family | MemberAttributes.Abstract;
            result.Name = "Accept";
            result.Parameters.Add(visitorParameter);
            return result;
        }

        public static CodeMemberMethod GetAcceptedMethod(string visitorClassName, string acceptedMethodName)
        {
            CodeMemberMethod result = new CodeMemberMethod();
            CodeParameterDeclarationExpression visitorParameter = new CodeParameterDeclarationExpression();
            visitorParameter.Name = "visitor";
            visitorParameter.Type = new CodeTypeReference(visitorClassName);
            result.Attributes = MemberAttributes.Public | MemberAttributes.Final;
            result.Name = "Accepted";
            result.Parameters.Add(visitorParameter);
            result.Statements.Add(CodeTypeMemberBuilder.CreateArgumentNullCheckStatement(visitorParameter, true));
            result.Statements.Add(new CodeMethodInvokeExpression(new CodeThisReferenceExpression(), acceptedMethodName, new CodeArgumentReferenceExpression(visitorParameter.Name)));
            return result;
        }

    }
}
