﻿// <copyright file="ArrayHelper.cs" company="Felipe José Angriman">
//   Copyright 2009 Felipe José Angriman 
//
//   Licensed under the Apache License, Version 2.0 (the "License"); 
//   you may not use this file except in compliance with the License. 
//   You may obtain a copy of the License at 
//   
//       http://www.apache.org/licenses/LICENSE-2.0 
// 
//   Unless required by applicable law or agreed to in writing, software
//   distributed under the License is distributed on an "AS IS" BASIS, 
//   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//   See the License for the specific language governing permissions and 
//   limitations under the License. 
// </copyright>
// <author>Felipe José Angriman</author>
// <summary>Defines the ArrayHelper Class</summary>

namespace Core.Collections
{
    using System;
    using System.Collections.Generic;
    using System.Text;

    /// <summary>
    /// A class that contains static methods for arrays.
    /// </summary>
    public static partial class ArrayHelper
    {
        /// <summary>
        /// A class that contains static methods for arrays that do not validate input.
        /// </summary>
        internal static partial class NonValidating
        {
            /// <summary>
            /// Fills an array with the items of an IEnumerable object.
            /// </summary>
            /// <typeparam name="T">The type of the items.</typeparam>
            /// <param name="destinationArray">The array to be filled.</param>
            /// <param name="sourceEnumerable">The IEnumerable object source of items.</param>
            internal static void FillFrom<T>(T[] destinationArray, IEnumerable<T> sourceEnumerable)
            {
                int index = 0;
                foreach (T item in sourceEnumerable)
                {
                    destinationArray[index] = item;
                    index++;
                }
            }

            /// <summary>
            /// Creates a new array filled with the items of an IEnumerable object.
            /// </summary>
            /// <typeparam name="T">The type of the objects in the IEnumerable object.</typeparam>
            /// <param name="enumerable">The IEnumerable object.</param>
            /// <returns>An array filled with the elements of the IEnumerable object.</returns>
            internal static T[] NewArrayFrom<T>(IEnumerable<T> enumerable)
            {
                T[] result = new T[IEnumerableHelper.Count(enumerable)];
                NonValidating.FillFrom(result, enumerable);
                return result;
            }
        }
    }
}
