﻿// <copyright file="DictionaryHelper.cs" company="Felipe José Angriman">
//   Copyright 2009 Felipe José Angriman 
//
//   Licensed under the Apache License, Version 2.0 (the "License"); 
//   you may not use this file except in compliance with the License. 
//   You may obtain a copy of the License at 
//   
//       http://www.apache.org/licenses/LICENSE-2.0 
// 
//   Unless required by applicable law or agreed to in writing, software
//   distributed under the License is distributed on an "AS IS" BASIS, 
//   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//   See the License for the specific language governing permissions and 
//   limitations under the License. 
// </copyright>
// <author>Felipe José Angriman</author>
// <summary>Defines the DictionaryHelper Class</summary>

namespace Core.Collections
{
    using System.Collections.Generic;

    /// <summary>
    /// A class that provides static methods for construction and manipulation of Dictionaries.
    /// </summary>
    public static class DictionaryHelper
    {
        /// <summary>
        /// Creates a dictionary from an IEnumerable of KeyValuePair.
        /// </summary>
        /// <typeparam name="TKey">The type of the keys.</typeparam>
        /// <typeparam name="TValue">The type of the values.</typeparam>
        /// <param name="pairs">The IEnumerable of KeyValuePair.</param>
        /// <returns>A dictionary with keys and values.</returns>
        public static Dictionary<TKey, TValue> NewDictionaryFrom<TKey, TValue>(IEnumerable<KeyValuePair<TKey, TValue>> pairs)
        {
            Dictionary<TKey, TValue> result = new Dictionary<TKey, TValue>();
            foreach (var pair in pairs)
            {
                result.Add(pair.Key, pair.Value);
            }

            return result;
        }
    }
}
