﻿// <copyright file="ImmutableReverseString.cs" company="Felipe José Angriman">
//   Copyright 2009 Felipe José Angriman 
//
//   Licensed under the Apache License, Version 2.0 (the "License"); 
//   you may not use this file except in compliance with the License. 
//   You may obtain a copy of the License at 
//   
//       http://www.apache.org/licenses/LICENSE-2.0 
// 
//   Unless required by applicable law or agreed to in writing, software
//   distributed under the License is distributed on an "AS IS" BASIS, 
//   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//   See the License for the specific language governing permissions and 
//   limitations under the License. 
// </copyright>
// <author>Felipe José Angriman</author>
// <summary>Defines the ImmutableReverseString Class</summary>

namespace Core.Collections
{
    using System.Collections.Generic;

    /// <summary>
    /// The reverse of an ImmutableString object.
    /// </summary>
    /// <typeparam name="T">The type of the items in the string.</typeparam>
    public sealed class ImmutableReverseString<T> : ImmutableString<T>
    {
        /// <summary>
        /// Holds the reversed ImmutableString object.
        /// </summary>
        private readonly ImmutableString<T> @string;

        /// <summary>
        /// Initializes a new instance of the ImmutableConcatenationString class.
        /// </summary>
        /// <param name="string">The ImmutableString object to reverse.</param>
        public ImmutableReverseString(ImmutableString<T> @string)
        {
            this.@string = @string;
        }

        /// <summary>
        /// Gets the reversed string.
        /// </summary>
        public ImmutableString<T> String
        {
            get
            {
                return this.@string;
            }
        }

        /// <summary>
        /// Gets the item at the given location.
        /// </summary>
        /// <param name="index">The zero-based index of the item to get.</param>
        /// <returns>The item at the given location.</returns>
        public override T GetItemAt(int index)
        {
            return this.@string[this.@string.Length - 1 - index];
        }

        /// <summary>
        /// Gets a typed enumerator for this ImmutableString object.
        /// </summary>
        /// <returns>A typed Enumerator for this ImmutableString object.</returns>
        public override IEnumerator<T> GetEnumerator()
        {
            for (int i = this.@string.Length - 1; i >= 0; i--)
            {
                yield return this.@string[i];
            }
        }
    }
}
