﻿// <copyright file="ImmutableSubstringString.cs" company="Felipe José Angriman">
//   Copyright 2009 Felipe José Angriman 
//
//   Licensed under the Apache License, Version 2.0 (the "License"); 
//   you may not use this file except in compliance with the License. 
//   You may obtain a copy of the License at 
//   
//       http://www.apache.org/licenses/LICENSE-2.0 
// 
//   Unless required by applicable law or agreed to in writing, software
//   distributed under the License is distributed on an "AS IS" BASIS, 
//   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//   See the License for the specific language governing permissions and 
//   limitations under the License. 
// </copyright>
// <author>Felipe José Angriman</author>
// <summary>Defines the ImmutableSubstringString Class</summary>

namespace Core.Collections
{
    using System;
    using System.Collections.Generic;

    /// <summary>
    /// 
    /// </summary>
    /// <typeparam name="T"></typeparam>
    public sealed class ImmutableSubstringString<T> : ImmutableString<T>
    {
        public ImmutableSubstringString(ImmutableString<T> @string, int start, int length)
        {
            if (@string == null)
            {
                throw new ArgumentNullException("string");
            }

            if (start < 0 || start >= @string.Length)
            {
                throw new ArgumentOutOfRangeException("start");
            }

            if (start + length >= @string.Length)
            {
                throw new ArgumentOutOfRangeException("length");
            }

            this.String = @string;
            this.SubstringStart = start;
            this.SubstringLength = length;
        }

        public ImmutableString<T> String
        {
            get;
            private set;
        }

        public int SubstringStart
        {
            get;
            private set;
        }

        public int SubstringLength
        {
            get;
            private set;
        }

        public override int Length
        {
            get
            {
                return this.SubstringLength;
            }
        }

        public override T GetItemAt(int index)
        {
            if (index < this.SubstringLength)
            {
                return this[this.SubstringStart + index];
            }
            else
            {
                throw new ArgumentOutOfRangeException("index");
            }
        }

        public override IEnumerator<T> GetEnumerator()
        {
            int lastItemIndex = this.SubstringStart + this.SubstringLength;
            for (int i = this.SubstringStart; i < lastItemIndex; i++)
            {
                yield return this[i];
            }
        }
    }
}
