﻿// <copyright file="IMutableSequence.cs" company="Felipe José Angriman">
//   Copyright 2009 Felipe José Angriman 
//
//   Licensed under the Apache License, Version 2.0 (the "License"); 
//   you may not use this file except in compliance with the License. 
//   You may obtain a copy of the License at 
//   
//       http://www.apache.org/licenses/LICENSE-2.0 
// 
//   Unless required by applicable law or agreed to in writing, software
//   distributed under the License is distributed on an "AS IS" BASIS, 
//   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//   See the License for the specific language governing permissions and 
//   limitations under the License. 
// </copyright>
// <author>Felipe José Angriman</author>
// <summary>Defines the IMutableSequence Class</summary>

namespace Core.Collections
{
    using System.Collections.Generic;

    /// <summary>
    /// A sequence that allows items to be added and removed.
    /// </summary>
    /// <typeparam name="T">The type of the items in the sequence.</typeparam>
    public interface IMutableSequence<T> : ISequence<T>
    {
        /// <summary>
        /// Adds an item to the sequence.
        /// </summary>
        /// <param name="item">The item to add.</param>
        void Add(T item);

        /// <summary>
        /// Adds items to the sequence.
        /// </summary>
        /// <param name="range">The items to add.</param>
        void AddRange(IEnumerable<T> range);

        /// <summary>
        /// Removes all elements from the sequence.
        /// </summary>
        void Clear();

        /// <summary>
        /// Removes an element from the sequence.
        /// </summary>
        /// <param name="item">The item to remove.</param>
        /// <returns>true if the item has been removed, false otherwise.</returns>
        bool Remove(T item);
    }
}
