﻿// <copyright file="IRandomAccessMutableSequence.cs" company="Felipe José Angriman">
//   Copyright 2009 Felipe José Angriman 
//
//   Licensed under the Apache License, Version 2.0 (the "License"); 
//   you may not use this file except in compliance with the License. 
//   You may obtain a copy of the License at 
//   
//       http://www.apache.org/licenses/LICENSE-2.0 
// 
//   Unless required by applicable law or agreed to in writing, software
//   distributed under the License is distributed on an "AS IS" BASIS, 
//   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//   See the License for the specific language governing permissions and 
//   limitations under the License. 
// </copyright>
// <author>Felipe José Angriman</author>
// <summary>Defines the IRandomAccessMutableSequence Class</summary>

namespace Core.Collections
{
    using System.Collections.Generic;

    /// <summary>
    /// A sequence that allows items to be added and removed at arbitrary positions.
    /// </summary>
    /// <typeparam name="T">The type of the items in the sequence.</typeparam>
    public interface IRandomAccessMutableSequence<T> : ISequence<T>
    {
        /// <summary>
        /// Inserts an item at a given location.
        /// </summary>
        /// <param name="item">The item to insert.</param>
        /// <param name="index">The location where to insert the item.</param>
        void InsertAt(T item, int index);

        /// <summary>
        /// Removes an item at the given location.
        /// </summary>
        /// <param name="index">The location where to remove the item.</param>
        void RemoveAt(int index);

        /// <summary>
        /// Inserts items at a given location.
        /// </summary>
        /// <param name="items">The items to insert.</param>
        /// <param name="index">The location where to insert the items.</param>
        void InsertRangeAt(IEnumerable<T> items, int index);

        /// <summary>
        /// Removes items at a given location.
        /// </summary>
        /// <param name="index">The location where to remove items.</param>
        /// <param name="length">The number of items ro remove.</param>
        void RemoveRangeAt(int index, int length);
    }
}
