﻿// <copyright file="ISequence.cs" company="Felipe José Angriman">
//   Copyright 2009 Felipe José Angriman 
//
//   Licensed under the Apache License, Version 2.0 (the "License"); 
//   you may not use this file except in compliance with the License. 
//   You may obtain a copy of the License at 
//   
//       http://www.apache.org/licenses/LICENSE-2.0 
// 
//   Unless required by applicable law or agreed to in writing, software
//   distributed under the License is distributed on an "AS IS" BASIS, 
//   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//   See the License for the specific language governing permissions and 
//   limitations under the License. 
// </copyright>
// <author>Felipe José Angriman</author>
// <summary>Defines the ISequence Class</summary>

namespace Core.Collections
{
    using System.Collections.Generic;

    /// <summary>
    /// A sequence of items with known length.
    /// </summary>
    /// <typeparam name="T">The type of the items in the sequence.</typeparam>
    public interface ISequence<T> : IEnumerable<T>
    {
        /// <summary>
        /// Gets the number of items in this sequence.
        /// </summary>
        /// <value>The number of items in this sequence.</value>
        int Length
        {
            get;
        }

        /// <summary>
        /// Determinates if this Sequence Contains an item.
        /// </summary>
        /// <param name="item">The item to look for.</param>
        /// <returns>true if the items belongs to the sequence, false otherwise.</returns>
        bool Contains(T item);
    }
}
