﻿// <copyright file="NotifyMutableSequenceChangedEventArgs.cs" company="Felipe José Angriman">
//   Copyright 2009 Felipe José Angriman 
//
//   Licensed under the Apache License, Version 2.0 (the "License"); 
//   you may not use this file except in compliance with the License. 
//   You may obtain a copy of the License at 
//   
//       http://www.apache.org/licenses/LICENSE-2.0 
// 
//   Unless required by applicable law or agreed to in writing, software
//   distributed under the License is distributed on an "AS IS" BASIS, 
//   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//   See the License for the specific language governing permissions and 
//   limitations under the License. 
// </copyright>
// <author>Felipe José Angriman</author>
// <summary>Defines the NotifyMutableSequenceChangedEventArgs Class</summary>

namespace Core.Collections
{
    using System;
    
    /// <summary>
    /// Provides data for the SequenceChanged event. 
    /// </summary>
    /// <typeparam name="T">The type of the elements in the mutable sequence.</typeparam>
    public sealed class NotifyMutableSequenceChangedEventArgs<T> : EventArgs
    {
        /// <summary>
        /// Initializes a new instance of the NotityMutableSequenceChangedEventArgs class.
        /// </summary>
        /// <param name="action">The action that triggered the event.</param>
        /// <param name="newItems">The elements that have been added to or replaced in the sequence.</param>
        /// <param name="oldItems">The elements that have been removed from o replaced in the sequence.</param>
        public NotifyMutableSequenceChangedEventArgs(NotifyMutableSequenceChangedAction action, ImmutableString<NotifyMutableSequenceChangedItem<T>> newItems, ImmutableString<NotifyMutableSequenceChangedItem<T>> oldItems)
        {
            this.Action = action;
            this.NewItems = newItems;
            this.OldItems = oldItems;
        }

        /// <summary>
        /// Gets the action that triggered the event.
        /// </summary>
        /// <value>The action that triggered the event.</value>
        public NotifyMutableSequenceChangedAction Action { get; private set; }
        
        /// <summary>
        /// Gets the elements that have been added to or replaced in the sequence.
        /// </summary>
        /// <value>The elements that have been added to or replaced in the sequence.</value>
        public ImmutableString<NotifyMutableSequenceChangedItem<T>> NewItems { get; private set; }
        
        /// <summary>
        /// Gets the elements that have been removed from o replaced in the sequence.
        /// </summary>
        /// <value>The elements that have been removed from o replaced in the sequence.</value>
        public ImmutableString<NotifyMutableSequenceChangedItem<T>> OldItems { get; private set; }
    }
}
