﻿// <copyright file="ExceptionHelper.cs" company="Felipe José Angriman">
//   Copyright 2009 Felipe José Angriman 
//
//   Licensed under the Apache License, Version 2.0 (the "License"); 
//   you may not use this file except in compliance with the License. 
//   You may obtain a copy of the License at 
//   
//       http://www.apache.org/licenses/LICENSE-2.0 
// 
//   Unless required by applicable law or agreed to in writing, software
//   distributed under the License is distributed on an "AS IS" BASIS, 
//   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//   See the License for the specific language governing permissions and 
//   limitations under the License. 
// </copyright>
// <author>Felipe José Angriman</author>
// <summary>Defines the ExceptionHelper Class</summary>

namespace Core
{
    using System;

    /// <summary>
    /// A static class that provides methods for Validation.
    /// </summary>
    internal static class ExceptionHelper
    {
        /// <summary>
        /// Tests if the value is less or equal than the value to compare, if it isn't
        /// this method throws an ArgumentException.
        /// </summary>
        /// <param name="paramName">The name of the parameter.</param>
        /// <param name="value">The value to test.</param>
        /// <param name="valueToCompare">The value to compare.</param>
        internal static void ValidateArgumentLessOrEqualThan(string paramName, int value, int valueToCompare)
        {
            if (!(value <= valueToCompare))
            {
                throw new ArgumentException(string.Empty, paramName);
            }
        }

        /// <summary>
        /// Tests if the value is greater than the value to compare, if it isn't
        /// this method throws an ArgumentException.
        /// </summary>
        /// <param name="paramName">The name of the parameter.</param>
        /// <param name="value">The value to test.</param>
        /// <param name="valueToCompare">The value to compare.</param>
        internal static void ValidateArgumentGreaterThan(string paramName, int value, int valueToCompare)
        {
            if (!(value > valueToCompare))
            {
                throw new ArgumentException(string.Empty, paramName);
            }
        }

    }
}
