﻿// <copyright file="TypeExpression.cs" company="Felipe José Angriman">
//   Copyright 2009 Felipe José Angriman 
//
//   Licensed under the Apache License, Version 2.0 (the "License"); 
//   you may not use this file except in compliance with the License. 
//   You may obtain a copy of the License at 
//   
//       http://www.apache.org/licenses/LICENSE-2.0 
// 
//   Unless required by applicable law or agreed to in writing, software
//   distributed under the License is distributed on an "AS IS" BASIS, 
//   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//   See the License for the specific language governing permissions and 
//   limitations under the License. 
// </copyright>
// <author>Felipe José Angriman</author>
// <summary>Defines the TypeExpression Class</summary>

namespace Core.Reflection.TypeExpressions
{
    using System;

    public abstract class TypeExpression
    {
        public void Accept(ITypeExpressionVisitor visitor)
        {
            if (visitor == null)
            {
                throw new ArgumentNullException("visitor");
            }
            this.Accepted(visitor);
        }

        protected abstract void Accepted(ITypeExpressionVisitor visitor);

        public static TypeAndExpression And(TypeBooleanExpression firstExpression, TypeBooleanExpression secondExpression)
        {
            return new TypeAndExpression(firstExpression, secondExpression);
        }
        public static TypeOrExpression Or(TypeBooleanExpression firstExpression, TypeBooleanExpression secondExpression)
        {
            return new TypeOrExpression(firstExpression, secondExpression);
        }
        public static TypeNotExpression Not(TypeBooleanExpression expression)
        {
            return new TypeNotExpression(expression);
        }
        public static TypeEqualsExpression Equals(TypeReferenceExpression firstExpression, TypeReferenceExpression secondExpression)
        {
            return new TypeEqualsExpression(firstExpression, secondExpression);
        }
        public static TypeSubtypeExpression Subtype(TypeReferenceExpression firstExpression, TypeReferenceExpression secondExpression)
        {
            return new TypeSubtypeExpression(firstExpression, secondExpression);
        }
        public static TypeConstantReferenceExpression ConstantReference(Type constant)
        {
            return new TypeConstantReferenceExpression(constant);
        }
        public static TypeVariableReferenceExpression VariableReference(string variable)
        {
            return new TypeVariableReferenceExpression(variable);
        }
        public static TypeIsGenericExpression IsGeneric(TypeReferenceExpression firstExpression, TypeReferenceExpression secondExpression)
        {
            return new TypeIsGenericExpression(firstExpression, secondExpression);
        }
    }
}
